import {
  Badge,
  Input,
  Popconfirm,
  Space,
  Table,
  Typography,
  notification,
} from "antd";
import { useTranslation } from "react-i18next";
import { useRoboticsRobotContext } from "../../../Contexts/RoboticsRobot";
import { useEffect, useRef, useState } from "react";
import {
  Constants,
  FormatDatetime,
  myFetch,
  myFetchContentType,
} from "../../../utils";
import MyPagination from "../../../Components/MyPagination";
import { Link } from "react-router-dom";

const ReceivedSSECommands = {
  UpdateRobotStatus: 1,
  AddUnauthorizedRobot: 2,
  AddRobot: 3,
  RemoveUnauthorizedRobot: 4,
  RemoveRobot: 5,
  RobotUpdated: 6,
};

function getRobotTypeString(type) {
  switch (type) {
    case 1:
      return "Rex";
    case 2:
      return "Yeet";
    default:
      return "Unknown";
  }
}

export default function Robots() {
  const robotsContext = useRoboticsRobotContext();
  const { t } = useTranslation();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();

  const [robotsPaginationPage, setRobotsPaginationPage] = useState(1);
  const [
    unauthorizedRobotsPaginationPage,
    setUnauthorizedRobotsPaginationPage,
  ] = useState(1);
  const [selectedRobotName, setSelectedRobotName] = useState("");

  const sseEventSource = useRef(null);

  const getRobotStatusBadge = (status) => {
    switch (status) {
      case 1:
        return (
          <Badge status="success" text={t("robotics.robots.status.idle")} />
        );
      case 2:
        return (
          <Badge
            status="processing"
            text={t("robotics.robots.status.processing")}
          />
        );
      case 3:
        return (
          <Badge
            status="warning"
            text={t("robotics.robots.status.connecting")}
          />
        );
      case 4:
        return (
          <Badge status="warning" text={t("robotics.robots.status.error")} />
        );
      case 5:
        return (
          <Badge status="default" text={t("robotics.robots.status.offline")} />
        );
      default:
        return "Unknown";
    }
  };

  const getRobotsTableContent = () => {
    let items = [
      {
        title: t("robotics.robots.column.id"),
        dataIndex: "id",
        key: "id",
      },
      {
        title: t("robotics.robots.column.type"),
        dataIndex: "type",
        key: "type",
      },
      {
        title: t("robotics.robots.column.name"),
        dataIndex: "name",
        key: "name",
      },
      {
        title: t("robotics.robots.column.status"),
        dataIndex: "status",
        key: "status",
      },
      {
        title: t("robotics.robots.column.currentJob"),
        dataIndex: "currentJob",
        key: "currentJob",
      },
      {
        title: t("robotics.robots.column.jobsWaiting"),
        dataIndex: "jobsWaiting",
        key: "jobsWaiting",
      },
      {
        title: t("robotics.robots.column.address"),
        dataIndex: "address",
        key: "address",
      },
      {
        title: t("robotics.robots.column.connectedAt"),
        dataIndex: "connectedAt",
        key: "connectedAt",
      },
      {
        title: t("robotics.robots.column.firmwareVersion"),
        dataIndex: "firmwareVersion",
        key: "firmwareVersion",
      },
      {
        title: t("robotics.robots.column.createdAt"),
        dataIndex: "createdAt",
        key: "createdAt",
      },
      {
        title: t("robotics.robots.column.actions"),
        dataIndex: "actions",
        key: "actions",
        render: (_, record) => (
          <Space size="middle">
            <Popconfirm
              placement="left"
              title={t("robotics.robots.popconfirmEdit.title")}
              description={
                <Input
                  placeholder="Name"
                  value={selectedRobotName}
                  onChange={(e) => setSelectedRobotName(e.target.value)}
                  minLength={Constants.GLOBALS.MIN_ROBOTICS_ROBOT_NAME_LENGTH}
                  maxLength={Constants.GLOBALS.MAX_ROBOTICS_ROBOT_NAME_LENGTH}
                />
              }
              okButtonProps={{
                disabled:
                  selectedRobotName.length <
                    Constants.GLOBALS.MIN_ROBOTICS_ROBOT_NAME_LENGTH ||
                  selectedRobotName.length >
                    Constants.GLOBALS.MAX_ROBOTICS_ROBOT_NAME_LENGTH,
              }}
              okText={t("common.button.confirm")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                myFetch(
                  `/robot`,
                  "PATCH",
                  {
                    robotId: record.id,
                    name: selectedRobotName,
                  },
                  {},
                  myFetchContentType.JSON,
                  Constants.ROBOTICS_API_ADDRESS
                ).catch((err) => {
                  if (err === 422) {
                    notificationApi["error"]({
                      message: t(
                        "robotics.robots.popconfirmEdit.errorNotification.message"
                      ),
                      description: t(
                        "robotics.robots.popconfirmEdit.errorNotification.description"
                      ),
                    });
                  }
                })
              }
            >
              <Link to="#" onClick={() => setSelectedRobotName(record.name)}>
                {t("common.text.edit")}
              </Link>
            </Popconfirm>

            <Popconfirm
              placement="left"
              title={t("robotics.robots.popconfirmDisconnect.title")}
              description={t(
                "robotics.robots.popconfirmDisconnect.description"
              )}
              okText={t("common.button.confirm")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                myFetch(
                  `/robot/${record.id}`,
                  "DELETE",
                  null,
                  {},
                  myFetchContentType.JSON,
                  Constants.ROBOTICS_API_ADDRESS
                )
              }
            >
              <Link to="#">{t("common.text.disconnect")}</Link>
            </Popconfirm>
          </Space>
        ),
      },
    ];

    return items;
  };

  const getRobotsTableItems = (robots) => {
    let items = [];

    robots.sort((a, b) => a.Status - b.Status);

    robots.forEach((robot) =>
      items.push({
        key: robot.Id,
        id: robot.Id,
        type: getRobotTypeString(robot.Type),
        name: robot.Name,
        status: getRobotStatusBadge(robot.Status),
        currentJob:
          robot.CurrentJobId === ""
            ? Constants.TEXT_EMPTY_PLACEHOLDER
            : robot.CurrentJobId,
        jobsWaiting: robot.JobsWaitingCount,
        address: robot.Address,
        firmwareVersion: robot.FirmwareVersion,
        connectedAt: FormatDatetime(robot.ConnectedAt),
        createdAt: FormatDatetime(robot.CreatedAt),
        actions: robot.Actions,
      })
    );

    return items;
  };

  const getUnauthorizedTableContent = () => {
    let items = [
      {
        title: t("robotics.robots.column.id"),
        dataIndex: "id",
        key: "id",
      },
      {
        title: t("robotics.robots.column.type"),
        dataIndex: "type",
        key: "type",
      },
      {
        title: t("robotics.robots.column.address"),
        dataIndex: "address",
        key: "address",
      },
      {
        title: t("robotics.robots.column.connectedAt"),
        dataIndex: "connectedAt",
        key: "connectedAt",
      },
      {
        title: t("robotics.robots.column.firmwareVersion"),
        dataIndex: "firmwareVersion",
        key: "firmwareVersion",
      },
      {
        title: t("robotics.robots.column.actions"),
        dataIndex: "actions",
        key: "actions",
        render: (_, record) => (
          <Space size="middle">
            <Popconfirm
              placement="left"
              title={t("robotics.unauthorizedRobots.popconfirmDeny.title")}
              description={t(
                "robotics.unauthorizedRobots.popconfirmDeny.description"
              )}
              okText={t("common.button.confirm")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                myFetch(
                  `/robot/deny/${record.id}`,
                  "DELETE",
                  null,
                  {},
                  myFetchContentType.JSON,
                  Constants.ROBOTICS_API_ADDRESS
                )
              }
            >
              <Link to="#">{t("common.text.deny")}</Link>
            </Popconfirm>

            <Popconfirm
              placement="left"
              title={t("robotics.unauthorizedRobots.popconfirmAuthorize.title")}
              description={t(
                "robotics.unauthorizedRobots.popconfirmAuthorize.description"
              )}
              okText={t("common.button.confirm")}
              cancelText={t("common.button.cancel")}
              onConfirm={() =>
                myFetch(
                  `/robot/authorize/${record.id}`,
                  "POST",
                  null,
                  {},
                  myFetchContentType.JSON,
                  Constants.ROBOTICS_API_ADDRESS
                )
              }
            >
              <Link to="#">{t("common.text.authorize")}</Link>
            </Popconfirm>
          </Space>
        ),
      },
    ];

    return items;
  };

  const getUnauthorizedTableItems = (unauthorizedRobots) => {
    let items = [];

    unauthorizedRobots.forEach((robot) => {
      items.push({
        key: robot.Id,
        id: robot.Id,
        type: getRobotTypeString(robot.Type),
        address: robot.Address,
        connectedAt: FormatDatetime(robot.ConnectedAt),
        firmwareVersion: robot.FirmwareVersion,
        actions: robot.Actions,
      });
    });

    return items;
  };

  // type = 0 => robots, type = 1 => unauthorizedRobots
  const fetchRobots = (type, page = 1) => {
    myFetch(
      `/${type === 1 ? "u" : ""}robots?page=${page}`,
      "GET",
      null,
      {},
      myFetchContentType.JSON,
      Constants.ROBOTICS_API_ADDRESS
    ).then((data) => {
      if (type === 1) {
        robotsContext.setUnauthorizedRobots(
          data.UnauthorizedRobots === null ? [] : data.UnauthorizedRobots
        );

        robotsContext.setUnauthorizedRobotsTotalPages(data.TotalPages);
      } else {
        robotsContext.setRobots(data.Robots === null ? [] : data.Robots);
        robotsContext.setRobotsTotalPages(data.TotalPages);
      }
    });
  };

  useEffect(() => {
    fetchRobots(0);

    fetchRobots(1);

    sseEventSource.current = new EventSource(
      `${Constants.ROBOTICS_API_ADDRESS}/sse`
    );

    sseEventSource.current.onmessage = (event) => {
      const data = JSON.parse(event.data);

      const cmd = data.Cmd;
      const body = data.Body;

      console.log("sse message", data);

      switch (cmd) {
        case ReceivedSSECommands.UpdateRobotStatus:
          robotsContext.setRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body.RobotId);

            if (index !== -1) {
              newArr[index].Status = body.Status;
            }

            return newArr;
          });
          break;
        case ReceivedSSECommands.AddUnauthorizedRobot:
          robotsContext.setUnauthorizedRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body.Id);

            if (index !== -1) {
              newArr[index] = body;
            } else {
              newArr.push(body);
            }

            return newArr;
          });
          break;
        case ReceivedSSECommands.AddRobot:
          robotsContext.setRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body.Id);

            if (index !== -1) {
              newArr[index] = body;
            } else {
              newArr.push(body);
            }

            return newArr;
          });

          // remove from unauthorized robots
          robotsContext.setUnauthorizedRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body.Id);

            if (index !== -1) {
              newArr.splice(index, 1);
            }

            return newArr;
          });
          break;
        case ReceivedSSECommands.RemoveUnauthorizedRobot:
          robotsContext.setUnauthorizedRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body);

            if (index !== -1) {
              newArr.splice(index, 1);
            }

            return newArr;
          });
          break;
        case ReceivedSSECommands.RemoveRobot:
          robotsContext.setRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body);

            if (index !== -1) {
              newArr.splice(index, 1);
            }

            return newArr;
          });
          break;
        case ReceivedSSECommands.RobotUpdated:
          robotsContext.setRobots((arr) => {
            const newArr = [...arr];

            const index = arr.findIndex((x) => x.Id === body.RobotId);

            if (index !== -1) {
              newArr[index].Name = body.Name;
            }

            return newArr;
          });
          break;
        default:
          break;
      }
    };

    sseEventSource.current.onerror = (event) => console.log("sse error", event);

    sseEventSource.current.onopen = (event) => console.log("sse open", event);

    sseEventSource.current.onclose = (event) => console.log("sse close", event);

    return () => sseEventSource.current.close();
  }, []);

  return (
    <>
      {notificationContextHolder}

      <Typography.Title level={4}>
        {t("robotics.robots.header")} ({robotsContext.robots.length})
      </Typography.Title>

      <Table
        scroll={{ x: "max-content" }}
        columns={getRobotsTableContent()}
        dataSource={getRobotsTableItems(robotsContext.robots)}
        pagination={false}
      />

      <MyPagination
        paginationPage={robotsPaginationPage}
        setPaginationPage={(page) => setRobotsPaginationPage(page)}
        totalPages={robotsContext.robotsTotalPages}
      />

      <Typography.Title level={4}>
        {t("robotics.unauthorizedRobots.header")} (
        {robotsContext.unauthorizedRobots.length})
      </Typography.Title>

      <Table
        scroll={{ x: "max-content" }}
        columns={getUnauthorizedTableContent()}
        dataSource={getUnauthorizedTableItems(robotsContext.unauthorizedRobots)}
        pagination={false}
      />

      <MyPagination
        paginationPage={unauthorizedRobotsPaginationPage}
        setPaginationPage={(page) => setUnauthorizedRobotsPaginationPage(page)}
        totalPages={robotsContext.unauthorizedRobotsTotalPages}
      />
    </>
  );
}
